#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Dec 17 17:55:19 2017

@author: Salim

Référence : Introduction to Machine Learning with Python, A. C. Muller et Sarah Guido, O'Reilly 2017
"""
import numpy as np
from sklearn.preprocessing import StandardScaler
from sklearn.datasets import load_breast_cancer
from sklearn.decomposition import PCA
import matplotlib.pyplot as plt
import seaborn as sns


cancer = load_breast_cancer()
scaler = StandardScaler()
scaler.fit(cancer.data)
X_scaled = scaler.transform(cancer.data)

# On spécifie le nombre de composantes à conserver
pca = PCA(n_components = 2)

# On détermine les composantes spécifiées
pca.fit(X_scaled)

# On projette les données sur les composantes spécifiées
X_pca = pca.transform(X_scaled)
print("Dimensions initiales : {}".format(str(X_scaled.shape)))
print("Dimensions réduites : {}".format(str(X_pca.shape)))

# Représentation graphique des données dans le plan factoriel
# coloriées par classe (malin/bénin)

sns.set(style="white")
plt.figure(figsize=(8,8))
colors = np.where(cancer.target==0,"b","r")
plt.scatter(X_pca[:,0], X_pca[:,1], c=colors, alpha = 0.5)
plt.xlabel("PC1")
plt.ylabel("PC2")
plt.show()