#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Nov 25 19:52:53 2017

@author: Salim
"""

###############################
# Quelques graphiques Python  #
###############################

# Bibliographie

# An Introduction to Statistics with Python - Thomas Halswanter - Springer 2016

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import scipy.stats as stats
import seaborn as sns

# Stripplot

x = np.random.randn(500)
plt.plot(x,'.')
plt.show()

# Histogramme

plt.hist(x,bins=25)

# Estimation de la densité par noyaux

sns.kdeplot(x)

# Fréquences cumulées
plt.plot(stats.cumfreq(x)[0])

# Boxplot

plt.boxplot(x,sym='*')

# Violin plots

nd = stats.norm
data = nd.rvs(size=(100))
nd2 = stats.norm(loc=3,scale=1.5)
data2 = nd2.rvs(size=(100))
df = pd.DataFrame({'Girls':data,'Boys':data2})
sns.violinplot(data=df)

# Diagrammes en barres empilés

df = pd.DataFrame(np.random.rand(10,4),columns = ['a','b','c','d'])
df.plot(kind='bar',grid=False)

# Diagramme en secteurs (camembert)

import seaborn as sns
import matplotlib.pyplot as plt
txtLabels = 'Cats', 'Dogs', 'Frogs', 'Others'
fractions = [45,30,15,10]
offsets=(0,0.05,0,0)
plt.pie(fractions, explode=offsets,labels=txtLabels,autopct='%1.1f%%', shadow=True,startangle=90
        ,colors=sns.color_palette('muted'))
plt.axis('equal')

# Nuage de points

df2 = pd.DataFrame(np.random.rand(50,4),columns=['a','b','c','d'])
df2.plot(kind='scatter', x='a', y='b', s=df['c']*300)

# Graphique 3D

import numpy as np
from matplotlib import cm
from mpl_toolkits.mplot3d.axes3d import get_test_data
fig = plt.figure(figsize=plt.figaspect(0.5))
ax = fig.add_subplot(1,2,1, projection='3d')
X = np.arange(-5,5,0.1)
Y = np.arange(-5,5,0.1)
X, Y = np.meshgrid(X,Y)
R = np.sqrt(X**2+Y**2)
Z = np.sin(R)
surf = ax.plot_surface(X,Y,Z,rstride=1,cstride=1,cmap=cm.GnBu,  linewidth=0,antialiased=False)
ax.set_zlim3d(-1.01,1.01)
fig.colorbar(surf, shrink=0.5, aspect = 10)
ax = fig.add_subplot(1,2,2,projection='3d')
X,Y,Z = get_test_data(0.5)
ax.plot_wireframe(X,Y,Z,rstride=1,cstride=1)

# Sauvegarde du graphique 3D

outfile = '/Users/Salim/Desktop/3dGraph.png'
plt.savefig(outfile, dpi=600)
print('Image sauvegardée sous {0}'.format(outfile))
plt.show()
