#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  7 16:02:15 2018

@author: Salim
"""
from sklearn.linear_model import LinearRegression
from sklearn.datasets import load_boston
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt
import numpy as np
from sklearn.preprocessing import MinMaxScaler, PolynomialFeatures

##############
# BOSTON
##############

boston = load_boston()

data = boston.data

idt = np.random.choice(range(data.shape[0]),size=200,replace=False)

y = boston.target[idt]

X = boston.data[idt,12].reshape(-1,1)

X_train, X_test, y_train, y_test = train_test_split(X,y,random_state=0)

lr = LinearRegression().fit(X_train, y_train)

print("lr.coef_: {}".format(lr.coef_))
print("lr.intercept_: {}".format(lr.intercept_))

xmin = X.min()
xmax = X.max()

line = np.linspace(xmin,xmax, 100).reshape(-1, 1)

plt.figure(figsize=(8, 8))
plt.plot(line, lr.predict(line))
plt.plot(X_train, y_train, 'o',c="b")
plt.plot(X_test,y_test,'^',c="r")
ax = plt.gca()
ax.set_xlabel("LSTAT")
ax.set_ylabel("Target")
ax.legend(["model", "training data", "test data"], loc="best")

print("Training set score: {:.2f}".format(lr.score(X_train,y_train)))
print("Test set score: {:.2f}".format(lr.score(X_test,y_test)))

##################
# Extended BOSTON
##################

def load_extended_boston():
    boston = load_boston()
    X = boston.data

    X = MinMaxScaler().fit_transform(boston.data)
    X = PolynomialFeatures(degree=2, include_bias=False).fit_transform(X)
    return X, boston.target

X, y = load_extended_boston()

X_train, X_test, y_train, y_test = train_test_split(X,y,random_state=0)

lr = LinearRegression().fit(X_train, y_train)

print("Training set score: {:.2f}".format(lr.score(X_train,y_train)))
print("Test set score: {:.2f}".format(lr.score(X_test,y_test)))



