#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 26 11:39:21 2018

@author: Salim
"""

from sklearn.datasets import load_iris
from sklearn.model_selection import train_test_split
import pandas as pd
import pandas.plotting as pdpl
from sklearn.neighbors import KNeighborsClassifier
import numpy as np

iris_dataset = load_iris()

print("Keys of iris_dataset :\n{}".format(iris_dataset.keys()))

print(iris_dataset['DESCR'][:193]+"\n...")

print("Target names: {}".format(iris_dataset['target_names']))

print("Feature names: {}".format(iris_dataset['feature_names']))

print("Type of data: {}".format(type(iris_dataset['data'])))

print("Shape of data: {}".format(iris_dataset['data'].shape))

print("First five lines of data:\n{}".format(iris_dataset['data'][:5]))

print("Type of target: {}".format(type(iris_dataset['target'])))

print("Shape of target: {}".format(iris_dataset['target'].shape))

print("Target:\n{}".format(iris_dataset['target']))

X_train, X_test, y_train, y_test = train_test_split(iris_dataset['data'],
                                                    iris_dataset['target'],
                                                    random_state=0)

print("y_train shape: {}".format(y_train.shape))
print("y_test shape: {}".format(y_test.shape))

iris_dataframe = pd.DataFrame(X_train, columns = iris_dataset.feature_names)

grr = pdpl.scatter_matrix(iris_dataframe, c=y_train, figsize=(15,15), marker='o',
                        hist_kwds={'bins':20}, s=60, alpha = 0.8)


knn = KNeighborsClassifier(n_neighbors=1)

knn.fit(X_train, y_train)

X_new = np.array([[5, 2.9, 1, 0.2]])
print("X_new shape: {}".format(X_new.shape))

prediction = knn.predict(X_new)

print("Prediction: {}".format(prediction))

print("Predicted target name: {}".format(iris_dataset['target_names'][prediction]))

y_pred = knn.predict(X_test)

print("Test set predictions:\n {}".format(y_pred))

print("Test set score: {:.2f}".format(np.mean(y_pred==y_test)))

print("Test set score: {:.2f}".format(knn.score(X_test,y_test)))

from sklearn.datasets import load_breast_cancer

cancer = load_breast_cancer()

print("cancer.keys(): \n{}".format(cancer.keys()))

print("Shape of cancer data : {}".format(cancer.data.shape))

print("Sample count per class:\n{}".format(
        {n: v for n, v in zip(cancer.target_names,np.bincount(cancer.target))}))

print("Feature names: \n{}".format(cancer.feature_names))

from sklearn.datasets import load_boston

boston = load_boston()

print("Data shape: {}".format(boston.data.shape))

from sklearn.preprocessing import MinMaxScaler, PolynomialFeatures


def load_extended_boston():
    boston = load_boston()
    X = boston.data

    X = MinMaxScaler().fit_transform(boston.data)
    X = PolynomialFeatures(degree=2, include_bias=False).fit_transform(X)
    return X, boston.target

X, y = load_extended_boston()
print("X.shape: {}".format(X.shape))

from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import train_test_split
from sklearn.neighbors import KNeighborsClassifier
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap, colorConverter
import pandas as pd

cancer = load_breast_cancer()

df = pd.DataFrame(cancer['data'],columns=cancer['feature_names'])

df2 = pd.DataFrame(cancer['target'],columns=['target'])

df = pd.concat([df,df2],axis=1)


def plot_2d_separator(classifier, X,fill=False, ax=None, eps=None, alpha=1, 
                      linewidth=None, threshold=None,
                      cm= ListedColormap(['#0000aa', '#ff2020']),
                      linestyle="solid"):
    # binary?
    if eps is None:
        eps = X.std() / 2.

    if ax is None:
        ax = plt.gca()

    x_min, x_max = X[:, 0].min() - eps, X[:, 0].max() + eps
    y_min, y_max = X[:, 1].min() - eps, X[:, 1].max() + eps
    xx = np.linspace(x_min, x_max, 1000)
    yy = np.linspace(y_min, y_max, 1000)

    X1, X2 = np.meshgrid(xx, yy)
    X_grid = np.c_[X1.ravel(), X2.ravel()]
    try:
        decision_values = classifier.decision_function(X_grid)
        levels = [0] if threshold is None else [threshold]
        fill_levels = [decision_values.min()] + levels + [decision_values.max()]
    except AttributeError:
        # no decision_function
        decision_values = classifier.predict_proba(X_grid)[:, 1]
        levels = [.5] if threshold is None else [threshold]
        fill_levels = [0] + levels + [1]
    if fill:
        ax.contourf(X1, X2, decision_values.reshape(X1.shape),
                    levels=fill_levels, alpha=alpha,cmap=cm)        
    else:
        ax.contour(X1, X2, decision_values.reshape(X1.shape), levels=levels,
                   colors="black", alpha=alpha, linewidths=linewidth,
                   linestyles=linestyle, zorder=5)        
    ax.set_xlim(x_min, x_max)
    ax.set_ylim(y_min, y_max)
    ax.set_xticks(())
    ax.set_yticks(())


def discrete_scatter(x1, x2, y=None, markers=None, s=5, ax=None,
                     labels=None, padding=.2, alpha=1, c=None, markeredgewidth=None):
    if ax is None:
        ax = plt.gca()

    if y is None:
        y = np.zeros(len(x1))

    unique_y = np.unique(y)

    if markers is None:
        markers = ['o', '^', 'v', 'D', 's', '*', 'p', 'h', 'H', '8', '<', '>'] * 10

    if len(markers) == 1:
        markers = markers * len(unique_y)

    if labels is None:
        labels = unique_y

    # lines in the matplotlib sense, not actual lines
    lines = []

    current_cycler = mpl.rcParams['axes.prop_cycle']

    for i, (yy, cycle) in enumerate(zip(unique_y, current_cycler())):
        mask = y == yy
        # if c is none, use color cycle
        if c is None:
            color = cycle['color']
        elif len(c) > 1:
            color = c[i]
        else:
            color = c
        # use light edge for dark markers
        if np.mean(colorConverter.to_rgb(color)) < .4:
            markeredgecolor = "grey"
        else:
            markeredgecolor = "black"

        lines.append(ax.plot(x1[mask], x2[mask], markers[i], markersize=s,
                             label=labels[i], alpha=alpha, c=color,
                             markeredgewidth=markeredgewidth,
                             markeredgecolor=markeredgecolor)[0])

    if padding != 0:
        pad1 = x1.std() * padding
        pad2 = x2.std() * padding
        xlim = ax.get_xlim()
        ylim = ax.get_ylim()
        ax.set_xlim(min(x1.min() - pad1, xlim[0]), max(x1.max() + pad1, xlim[1]))
        ax.set_ylim(min(x2.min() - pad2, ylim[0]), max(x2.max() + pad2, ylim[1]))

    return lines

idt = np.random.choice(range(df.shape[0]),size=50,replace=False)


for n in range(1,11):

    clf = KNeighborsClassifier(n_neighbors=n)

    y_train = df.target[idt]
    X_train = df.iloc[idt,0:2]

    clf.fit(X_train,y_train)

    X = np.array(X_train)
    y = np.array(y_train)

    plot_2d_separator(clf,X,fill=True,eps=0.5,alpha=.4)
    discrete_scatter(X[:, 0], X[:, 1], y)
    plt.xlabel(cancer.feature_names[0])
    plt.ylabel(cancer.feature_names[1])
    plt.title("n = "+str(n))
    plt.show()
    
    
from sklearn.neighbors import KNeighborsRegressor
from sklearn.datasets import load_boston
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt

boston = load_boston()

data = boston.data

idt = np.random.choice(range(data.shape[0]),size=40,replace=False)

y = boston.target[idt]

X = boston.data[idt,12].reshape(-1,1)

X_train, X_test, y_train, y_test = train_test_split(X,y,random_state=0)

fig, axes = plt.subplots(1, 3, figsize=(15,4))

Xmin = X.min()
Xmax = X.max()

line = np.linspace(Xmin,Xmax,1000).reshape(-1,1)

for n_neighbors, ax in zip([1, 3, 9], axes):
    reg = KNeighborsRegressor(n_neighbors=n_neighbors)
    reg.fit(X_train,y_train)
    ax.plot(line,reg.predict(line))
    ax.plot(X_train,y_train,'^',markersize=8)
    ax.plot(X_test,y_test,'v',markersize=8)
    
    ax.set_title(
            "{} neighbors\n train score : {:.2f} test score: {:.2f}".format(
                    n_neighbors, reg.score(X_train,y_train),
                    reg.score(X_test,y_test)))
    ax.set_xlabel("LSTAT")
    ax.set_ylabel("target")
    
axes[0].legend(["Model predictions","Training data/target",
    "Test data/target"],loc="best")
            









